import datetime

DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
DAY_FORMAT = "%Y-%m-%d"


def get_each_day(start_time, end_time):
    """
    list<datetime> where (start_time <= datetime <= end_time)
    :param start_time:
    :param end_time:
    :return: list<datetime>
    """
    day_list = []
    while end_time >= start_time:
        day_list.append(start_time)
        start_time = start_time + datetime.timedelta(days=1)
    return day_list


def get_now():
    """
    当前时间
    :return: datetime
    """
    return datetime.datetime.now()


def get_time_by_offset(time, h_offset=0, m_offset=0, s_offset=0):
    """
    获取time的前后n小时/分钟/秒
    :param time: datetime
    :param h_offset: hour offset
    :param m_offset: minute offset
    :param s_offset: second offset
    :return: datetime
    """
    new_time = time + datetime.timedelta(hours=h_offset)
    new_time = new_time + datetime.timedelta(minutes=m_offset)
    new_time = new_time + datetime.timedelta(seconds=s_offset)
    return new_time


def get_date_by_offset(date, D_offset=0, M_offset=0, Y_offset=0):
    """
    获取date的前后n日/月/年
    :param date:
    :param D_offset: Day offset
    :param M_offset: Month offset
    :param Y_offset: Year offset
    :return: datetime
    """
    old_year = date.year
    old_month = date.month
    old_day = date.day
    new_year = old_year + Y_offset
    new_month = old_month + M_offset
    if new_month > 12:
        new_year += 1
        new_month -= 12
    new_day = old_day + D_offset
    new_date = date.replace(new_year, new_month, new_day)
    return new_date


def timestamp2str(timestamp):
    """
    :param timestamp:
    :return: str
    """
    try:
        d = datetime.datetime.fromtimestamp(timestamp)
        return d.strftime("%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(e)


if __name__ == '__main__':
    thelist = get_each_day(datetime.datetime.strptime(
        '2017-12-01 01:00:00', DATETIME_FORMAT), datetime.datetime.now())
    for i in thelist:
        print(i.strftime(DATETIME_FORMAT))
    print((get_date_by_offset(datetime.datetime.now(), 1, 1, 1) - get_now()).days)
    print(timestamp2str(1524878239))



'''
SSH提供了2种方式:
    1. 直接执行函数exec_onecmd,exec_multicmd,exec_cmdlist等
    2. new个SSH对象

eg:
exec_cmd('192.168.75.124','root','123456','ifconfig')
exec_cmds('192.168.75.124','root','123456',['cd /home','ls -l'])

ssh1 = SSH('192.168.75.124','root','123456')
ssh1.exec_cmd('ifconfig')
ssh1.exec_cmds(['cd /home','ls -l'])

newstr1 = list_to_cmd(['cd /home', 'ls -l'])
newstr2 = multistr_to_cmd('cd /home', 'ls -l')
'''

import paramiko


def exec_cmd(hostaddr, username, password, cmd):
    """
    执行一条命令
    :param hostaddr:
    :param username:
    :param password:
    :param cmd:
    :return:
    """
    s = paramiko.SSHClient()
    # 读取know_host
    s.load_system_host_keys()
    # 建立SSH连接
    s.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    s.connect(hostaddr, 22, username, password)
    stdin, stdout, stderr = s.exec_command(cmd)
    ret_list = []
    for x in stdout.readlines():
        ret_list.append(x.strip())
    s.close()
    return ret_list


def exec_cmds(hostaddr, username, password, cmdlist):
    """
    执行多条命令
    :param hostaddr:
    :param username:
    :param password:
    :param cmdlist:
    :return:
    """
    s = paramiko.SSHClient()
    s.load_system_host_keys()
    s.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    s.connect(hostaddr, 22, username, password)
    ret_list = []
    for cmd in cmdlist:
        stdin, stdout, stderr = s.exec_command(cmd)
        for x in stdout.readlines():
            ret_list.append(x.strip())
    s.close()
    return ret_list


def list_to_string(ret_list):
    ret_str = ''
    for ret in ret_list:
        ret_str += ret + '\n'
    return ret_str


def __list_to_cmd(strlist):
    """
    ['aa', 'bb', 'cc'] ==> 'aa;bb;cc;'
    :param strlist:
    :return:
    """
    temp_str = ''
    for x in strlist:
        temp_str = temp_str + x + ';'
    return temp_str


def __multistr_to_cmd(*strs):
    """
    ['aa', 'bb', 'cc'] ==> 'aa;bb;cc;'
    :param strs:
    :return:
    """
    temp_str = ''
    for x in strs:
        temp_str = temp_str + x + ';'
    return temp_str


class SSH():
    def __init__(self, hostaddr, username, password):
        super(SSH, self).__init__()
        self.sshclient = paramiko.SSHClient()
        self.sshclient.load_system_host_keys()
        self.sshclient.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.sshclient.connect(hostaddr, 22, username, password)

    def __del__(self):
        self.sshclient.close()

    def exec_cmd(self, cmd):
        stdin, stdout, stderr = self.sshclient.exec_command(cmd)
        ret_list = []
        for x in stdout.readlines():
            ret_list.append(x.strip())
        return ret_list

    def exec_cmds(self, cmdlist):
        for cmd in cmdlist:
            stdin, stdout, stderr = self.sshclient.exec_command(cmd)
            ret_list = []
            for x in stdout.readlines():
                ret_list.append(x.strip())
        return ret_list

    def close(self):
        self.sshclient.close()




def long_to_mac(value, connector='-'):
    raw_mac = hex(value)[2:].zfill(12).upper()
    ret_mac = ''
    for char, index in zip(raw_mac, range(len(raw_mac))):
        ret_mac += char
        if index & 1 == 1:
            ret_mac += connector
    return ret_mac[:-1]


if __name__ == '__main__':
    print(long_to_mac(19929521329))



# windows/linux--py2/3 均支持
import paramiko
import os


class FTP():
    """
    例子: ftp = FTP('39.1.222.119', 'root', '123456')
          ftp.upload_one("C:\\mysql_test.py", "/home/FTP_download/mysql_test.py")
          ftp.download_one("/home/FTP_downloads/quicksort.py", "C:\\Users\\linshuo.UT\\Desktop\\quicksort1.py")

    """

    def __init__(self, hostaddr, username, password):
        super(FTP, self).__init__()
        self.__trans = paramiko.Transport((hostaddr, 22))
        self.__trans.connect(username=username, password=password)
        self.sftp = paramiko.SFTPClient.from_transport(self.__trans)

    def __del__(self):
        self.__trans.close()

    def get_allfiles_in_localdir(self, localdir):
        """
        递归获取本地某文件夹下所有文件的完整路径
        :param localdir: 文件夹的路径
        :return:
        """
        allfiles = []
        files = os.listdir(localdir)
        for x in files:
            filename = os.path.join(localdir, x)
            if os.path.isdir(filename):
                allfiles.extend(self.get_allfiles_in_localdir(filename))
            else:
                allfiles.append(filename)
        return allfiles

    def upload_one(self, fromfile, tofile, callback=None):
        """
        上传一个文件
        :param fromfile: 上传文件的完整路径
        :param tofile: 放置文件的完整路径
        :parma callback: 回调(int, int)-->(so_far_size, total_size)
        :return:
        """
        self.sftp.put(localpath=fromfile, remotepath=tofile, callback=callback)
        (file_path, file_name) = os.path.split(fromfile)
        print(file_name, '上传完成!')

    def download_one(self, fromfile, tofile):
        """
        下载文件
        :param fromfile:
        :param tofile:
        :return:
        """
        self.sftp.get(localpath=tofile, remotepath=fromfile)
        (file_path, file_name) = os.path.split(tofile)
        print(file_name, '下载完成!')

    def upload_dir(self, localdir, remotedir):
        """
        上传localdir下所有文件包括子目录里, 但这些文件最后堆一起, 没有分文件夹放
        :param localdir:
        :param remotedir:
        :return:
        """
        if remotedir[-1] == '/':  # 去掉路径最后的'/', 如果有的话
            remotedir = remotedir[0:-1]
        allfiles = self.get_allfiles_in_localdir(localdir)
        for fullpath in allfiles:
            filename = os.path.split(fullpath)[-1]
            remote_fullpath = remotedir + '/' + filename
            print('传输成功:' + filename)
            self.upload_one(fullpath, remote_fullpath)

    def close(self):
        self.__trans.close()


if __name__ == '__main__':
    print('ftp_util')



from sqlalchemy import create_engine, text, Column, Integer, String, Sequence, DateTime, BigInteger, Text, Float, Date
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
import os


def get_sql_files():
    """
    获取当前目录的.sql文件名列表
    :return:
    """
    sql_files = filter(lambda x: x.endswith('.sql'), os.listdir(os.getcwd()))
    print(sql_files)


Base = declarative_base()


class User(Base):
    __tablename__ = 't_user'

    id = Column(Integer, Sequence('id'), primary_key=True)
    name = Column(String(64))
    gender = Column(String(64))
    age = Column(Integer)

    def __repr__(self):
        return "Product(id=%d, name=%s, gender=%s, age=%d)"(self.id, self.name, self.gender, self.age)


def sqlacodegen(dbname, outfile_name):
    """
    根据数据库逆向生成model.py
    :param dbname:
    :param outfile_name:
    :return:
    """
    cmd = 'sqlacodegen --noviews --noconstraints --noindexes --outfile ./%s.py mysql+pymysql://root:ut123456@localhost:3306/%s' % (
    outfile_name, dbname)
    ret = os.system(cmd)
    print(ret)


def __insert_demo(mysession):
    newuser = User()
    newuser.id = 3
    newuser.name = '小刚'
    newuser.gender = 'female'
    newuser.age = 12
    mysession.add(newuser)
    mysession.commit()
    mysession.close()


def __query_demo(mysession):
    res = mysession.query(User).filter(text("id=3")).one()
    print(res.id, res.name, res.gender, res.age)
    mysession.close()


def __update_demo(mysession):
    res = mysession.query(User).filter(text("id=3")).one()
    res.name = "大刚"
    mysession.commit()
    mysession.close()


def __delete_demo(mysession):
    res = mysession.query(User).filter(text("id=3")).one()
    mysession.delete(res)
    mysession.commit()
    mysession.close()


def get_mysql_session(user, password, db_name):
    """
    获取数据库连接
    :param user:
    :param password:
    :param db_name:
    :return:
    """
    DB_CON_STR = 'mysql+pymysql://%s:%s@localhost/%s?charset=utf8mb4' % (user, password, db_name)
    print(DB_CON_STR)
    engine = create_engine(DB_CON_STR, echo=False)
    DBSession = sessionmaker(bind=engine)
    return DBSession()


def execute_sqls(mysession, sqls):
    """
    执行完整的sql语句
    :param mysession:
    :param sqls:
    :return:
    """
    if isinstance(sqls, str):  # 单句sql直接执行
        mysession.execute(sqls)
    elif isinstance(sqls, list):  # 多句sql遍历执行
        for sql in sqls:
            mysession.execute(sql)
    elif isinstance(sqls, tuple):  # 多句sql遍历执行
        for sql in sqls:
            mysession.execute(sql)


if __name__ == '__main__':
    mysession = get_mysql_session('root', 'ut123456', 'testdatabase')
    __query_demo(mysession)
    # sqlacodegen('shortmsg','nodelnew')



import hashlib
import base64


def md5_bytes(s):
    """
    :param s:
    :return: bytes
    """
    h = hashlib.md5()
    h.update(s.encode())
    return h.digest()


def md5_string(s):
    """
    :param s:
    :return: str
    """
    h = hashlib.md5()
    h.update(s.encode())
    return h.hexdigest()


def sha1_bytes(s):
    """
    :param s:
    :return: bytes
    """
    h = hashlib.sha1()
    h.update(s.encode())
    return h.digest()  # bytes


def sha1_string(s):
    """
    :param s:
    :return: str
    """
    h = hashlib.sha1()
    h.update(s.encode())
    return h.hexdigest()  # str


def base64_encode(s):
    """
    :param s:
    return: str
    """
    bs = base64.urlsafe_b64encode(s.encode())  # bytes
    return bs.decode()


def base_decode(s):
    """
    :param s:
    :return: str
    """
    db = base64.urlsafe_b64decode(s)
    return db.decode()  # str



import requests


def request_json(method, url, data=None, headers=None, cookies=None):
    """
     eg:request("GET", "127.0.0.1:8000")
    :param method: 'GET' 'POST' 'PUT' 'DELETE'
    :param url: str
    :param data: dict
    :param headers: dict
    :param cookies: dict
    :return: (int, dict)
    """
    if not url.startswith('http://') and not url.startswith('https://'):
        url = 'http://' + url
    r = requests.request(method, url, data=data, headers=headers, cookies=cookies)
    print(url)
    print(r.status_code, r.json())
    if r.status_code != 200:
        return r.status_code, r.text
    return r.status_code, r.json()


def request(method, url, data=None, headers=None, cookies=None):
    """
     eg:request("GET", "127.0.0.1:8000")
    :param method: 'GET' 'POST' 'PUT' 'DELETE'
    :param url: str
    :param data: dict
    :param headers: dict
    :param cookies: dict
    :return: (int, str)
    """
    if not url.startswith('http://') and not url.startswith('https://'):
        url = 'http://' + url
    r = requests.request(method, url, data=data, headers=headers, cookies=cookies)
    print(url)
    print(r.status_code, r.text)
    return r.status_code, r.text


if __name__ == '__main__':
    request("GET", "127.0.0.1:8000/p/wechat/mp/jsapi/signature")



import sys
import os
import codecs
import natsort


def get_file_content(file_path):
    """
    读取文件, 暂时只支持utf8和gbk编码的文件, 自动去除BOM
    :param file_path:
    :return: str
    """
    try:
        with open(file_path, encoding='utf-8') as f1:
            raw = f1.read()
            # 去掉BOM
            bom_head = raw.encode(encoding='utf-8')[:3]
            if bom_head == codecs.BOM_UTF8:
                raw = raw.encode(encoding='utf-8')[3:].decode(encoding='utf-8')
            return raw
    except Exception as e:
        with open(file_path, encoding='GBK') as f2:
            return f2.read()


def write_file_content(file_path, text, encoding='utf8'):
    """
    写文件
    :param file_path: str
    :param text: str
    :param encoding: str
    :return: None
    """
    try:
        with open(file_path, mode='wb') as f1:
            f1.write(text.encode(encoding=encoding))
    except Exception as e:
        print(e)


def write_file_content_append(file_path, text, encoding='utf8'):
    """
    写文件-append模式
    :param file_path: str
    :param text: str
    :param encoding: str
    :return: None
    """
    try:
        with open(file_path, mode='ab') as f1:
            f1.write(text.encode(encoding=encoding))
    except Exception as e:
        print(e)


def quick_mkdir(name):
    """
    当前目录下建一个文件夹
    :param name: 文件夹名称
    :return: 新建的文件夹的完整路径
    """
    new_directory = os.getcwd() + '\\' + name + "\\"
    if not os.path.exists(new_directory):
        try:
            os.mkdir(os.getcwd() + '\\' + name)
        except Exception as e:
            print(e)
    return new_directory


def get_files_fullpath(dir_path, suffix=''):
    """
    获取dir_path目录下所有.xxx文件的路径
    :param suffix: 后缀如".sql" ".java" ; 若不填则不进行文件过滤
    :return: list of str
    """
    files = list(filter(lambda x: os.path.isfile(
        os.path.join(dir_path, x)), os.listdir(dir_path)))
    if suffix != '':
        # 留下后缀为suffix的文件
        files = list(filter(lambda x: x.endswith(suffix), files))
    all_fullpath = list(map(lambda x: os.path.join(dir_path, x), files))
    return all_fullpath


def get_files_fullpath_recur(dir_path, suffix=None):
    """
    获取dir_path目录及其子目录下所有.xxx文件的路径
    :param suffix: 后缀如".sql" ".java" ; 若不填则不进行文件过滤
    :return: str or list or tuple
    """
    if dir_path == '' or dir_path is None:
        return
    if os.path.isfile(dir_path):
        return
    urls = list()
    for maindir, subdir, filename_list in os.walk(dir_path):
        for file in filename_list:
            full_path = os.path.join(maindir, file)
            ext = os.path.splitext(full_path)[1]
            ok = False
            if suffix is None:
                ok = True
            elif isinstance(suffix, str):
                if suffix == ext:
                    ok = True
            elif isinstance(suffix, list) or isinstance(suffix, tuple):
                if suffix is None or ext in suffix:
                    ok = True
            if ok:
                urls.append(full_path)
    return urls


def get_files_fullpath_curdir(suffix=''):
    """
    获取当前目录下所有.xxx文件的路径
    :param suffix: 后缀如".sql" ".java" ; 若不填则不进行文件过滤
    :return: list of str
    """
    return get_files_fullpath(os.getcwd(), suffix)


def get_dirs_fullpath(dir_path):
    """
     获取dir_path目录下所有文件夹的路径
    """
    dirs = list(filter(lambda x: os.path.isdir(
        os.path.join(dir_path, x)), os.listdir(dir_path)))
    all_fullpath = list(map(lambda x: os.path.join(dir_path, x), dirs))
    return all_fullpath

def split_path_file_ext(path):
    """
    :return: (文件名, .扩展名)
    """
    path, filename = os.path.split(path)
    name, ext = os.path.splitext(filename)
    return path, name, ext

def path_with_tail(path, tail):
    """
    给文件名加后缀
    eg: newPath = path_with_tai('abc.jpg', '_1') # newPath为 'abc_1.jpg'
    """
    p, name, ext = split_path_file_ext(path)
    if p == '':
        os_sep = ''
    else:
        os_sep = os.path.sep
    return p + os_sep + name + tail + ext

if __name__ == '__main__':
    a = """D:\\git_project\\ShuoPyUtils\\shuogg\\des_util.py"""
    print(split_path_file_ext(a))



import jpype


def to_hex_str(value, byte_size):
    """
    10进制数转16进制字符串
    :param value:
    :param byte_size: 字节数,主要针对负数
    :return:
    """
    jvmPath = jpype.getDefaultJVMPath()
    jpype.startJVM(jvm=jvmPath)
    num2Hex_func = jpype.java.lang.Long.toHexString
    raw_str = num2Hex_func(value)[16 - 2 * byte_size:]
    return '0x' + raw_str.upper()

if __name__ == '__main__':
    print(to_hex_str(-25, 2))



from setuptools import setup

setup(
    name="shuogg",              # 包名
    version="0.1",              # 版本信息
    packages=['shuogg'],        # 要打包的项目文件夹
    description="shuoGG's common utils",
    keywords='python util',
    author='shuoGG',
    author_email='328893769@qq.com',
    url='https://github.com/shuoGG1239/ShuoPyUtils',
    zip_safe=False
)



import random
import uuid


def __num(n):
    """
    n个9的int
    :param n: 位数
    :return: int eg: num(2)-->99 num(3)-->999
    """
    return 10 ** (n) - 1


def numstr(n):
    """
    n位随机数的字符串
    :param n: 位数
    :return: str eg: numstr(2)-->'23' numstr(3)-->'551'
    """
    return str(random.randint(0, __num(n))).zfill(n)


def num(n):
    """
    n位随机整数
    :param n: 位数
    :return:
    """
    return int(numstr(n))


def numfloat(n, roundnum=2):
    """
    n位整数+roundnum位小数的浮点
    :param n:
    :param roundnum: 小数点后位数
    :return: float eg: floatnum(3)-->342.78
    """
    new_num = random.uniform(0, __num(n))
    return round(new_num, roundnum)


def mobile():
    """
    返回一个11位的手机号码-str
    return: str
    """
    mobile_rand = '1' + numstr(10)
    return mobile_rand


def email():
    email_types = ('@163.com', '@qq.com', '@sina.cn', '@sina.com')
    pre_rand = numstr(9)
    return pre_rand + random.choice(email_types)


def identification():
    """
    身份证
    :return:
    """
    id_rand = '44' + numstr(16)
    return id_rand


def list_gen(length, func, *arg):
    """
    list_gen(3, mobile)-->返回3个mobile结果
    :param length:
    :param func:
    :return: list
    """
    ret_list = []
    for x in range(length):
        ret_list.append(func(*arg))
    return ret_list


def uuid1():
    """
    16字节
    :return:
    """
    return str(uuid.uuid1()).replace('-', '')


if __name__ == '__main__':
    print(uuid1())
    print(list_gen(2, mobile))
    print(num(3))
